# coding: utf-8

require 'cairo'
require 'pango'
require 'RMagick'

#===============================================================================
#
#	Class: LegacyGraphics
#
class LegacyGraphics

	@@color = []
	pow = { '0' => 0, '1' => 1 }; (0..7).each {|c|				# イロイッカイズツ
		grb = '%03b' % c; @@color << [pow[grb[1, 1]], pow[grb[0, 1]], pow[grb[2, 1]]]
	}

	def self.get_image_size_png(file)
		@@png_image = Magick::Image.read(@@cur_png = file)
		[@@png_image[0].columns, @@png_image[0].rows]
	end

	def self.point(x, y)
		@@png_image[0].pixel_color(x, y)
	end

	def initialize(h_dmy, w_dmy, y_dmy, x_dmy, gy = 400, gx = 640, cy = 16, cx = 8, c = 0, out = {})
		@lx = 0
		@ly = 0

		@file = out[:file] || 'TLGCairo'; @ext = out[:type] || 'png'
		@ext =~ /png/i and @ext = 'png' and @surface = Cairo::ImageSurface.new(Cairo::FORMAT_ARGB32, gx, gy)
		@ext =~ /pdf/i and @ext = 'pdf' and @surface = Cairo::PDFSurface.new(    @file + '.' + @ext, gx, gy)
		@ext =~ /ps/i  and @ext = 'ps'  and @surface = Cairo::PSSurface.new(     @file + '.' + @ext, gx, gy)
		@context = Cairo::Context.new(@surface)

		fill(0, 0, gx - 1, gy - 1, c) if(c)
	end

	def pset(x, y, c)
		fill(x, y, x, y, c)
	end

	def locate(x, y)
		@tx = x * 8; @ty = y * 16
	end

	def print(mes)
		text(@tx, @ty, mes, 7)
	end

	def line(x1, y1, x2, y2, c, bf = '', w = 1, d = nil, a = nil)
		if(bf =~ /f/i)
			fill(x1, y1, x2, y2, c)
		elsif(bf =~ /b/i)
			polygon([[x1, y1], [x2, y1], [x2, y2], [x1, y2]], c)
		else
			@lx = x2
			@ly = y2
			_line(x1, y1, x2, y2, c, w, d, a)
		end
	end

	def cline(x, y, c, w = 1, d = nil, a = nil)
			  x1 = @lx
			  y1 = @ly
		@lx = x2 = x
		@ly = y2 = y
		_line(x1, y1, x2, y2, c, w, d, a)
	end

	#---------------------------------------------------------------------------
	#
	#	_line
	#
	def _line(x1, y1, x2, y2, c, w, d, a)

		@context.set_line_width(w)
		@context.set_source_rgb(*@@color[c.class <= Numeric ? c : 7])
		@context.move_to(x1, y1)
		@context.line_to(x2, y2)
		@context.stroke
	end

	def circle(x, y, r, c, f = '')

		@context.set_source_rgb(*@@color[c.class <= Numeric ? c : 7])
		@context.arc(x, y, r, 0, 2 * Math::PI)
		f =~ /f/i ? @context.fill : @context.stroke
	end

	def fill(x1, y1, x2, y2, c)

		@context.set_source_rgb(*@@color[c.class <= Numeric ? c : 7])
		@context.set_source_rgba(*c) if(c.class <= Array)
		@context.rectangle(x1, y1, x2 - x1 + 1, y2 - y1 + 1)
		@context.fill
	end

	def polygon(points, c, f = '')				# points = [[x1, y1], [x2, y2], [x3, y3]...]

		ps = points.dup
		@context.set_source_rgb(*@@color[c.class <= Numeric ? c : 7])
		@context.move_to(*(xy = ps.shift))
		@context.line_to(*xy) while(xy = ps.shift)
		@context.close_path
		f =~ /f/i ? @context.fill : @context.stroke
	end

	def put_image_png(x = 0, y = 0, file = @@cur_png)

		@context.set_source(Cairo::ImageSurface::from_png(@@cur_png = file), x, y)
		@context.paint
	end

	def text(x, y, s, c, ps = {})
#		{ 'font.family' => 'Helvetica', 'font.weight' => 400, 'font.size' => 10, 'layout.width' => 128, 'layout.alignment' => 'left', 'get_size' => true }

		# http://ruby-gnome2.sourceforge.jp/hiki.cgi?Pango%3A%3AFontDescription
		font = Pango::FontDescription.new
		font.family = ps[:font_family] || 'Helvetica'
		font.weight = ps[:font_weight] || Pango::WEIGHT_NORMAL	# WEIGHT_NORMAL: 400, WEIGHT_BOLD: 700
		font.absolute_size = (ps[:font_size] || 10) * Pango::SCALE

		# http://ruby-gnome2.sourceforge.jp/hiki.cgi?Pango%3A%3ALayout
		layout = @context.create_pango_layout
		layout.font_description = font

		layout.width = (ps[:layout_width] || 128) * Pango::SCALE	# x ------- width(dot) ->|
		layout.alignment = Pango::Layout::ALIGN_LEFT				# |left....center...right|
		layout.alignment = Pango::Layout::ALIGN_CENTER	if(it = ps[:layout_alignment] and it =~ /center/i)
		layout.alignment = Pango::Layout::ALIGN_RIGHT	if(it = ps[:layout_alignment] and it =~ /right/i)
		layout.text = s

		ps[:get_size] and return(layout.pixel_size)
		@context.set_source_rgb(*@@color[c.class <= Numeric ? c : 7])
		@context.move_to(x, y)
		@context.show_pango_layout(layout)
	end

	def rotate(r = nil, x = 0, y = 0)
		r and @context.translate(x, y) and @context.rotate( rad = r * Math::PI / 180) and @context.translate(-x, -y)
		yield
		r and @context.translate(x, y) and @context.rotate(-rad                     ) and @context.translate(-x, -y)
	end

	def clear
	end

	def box(vchar_dmy, hchar_dmy)
	end

	def refresh
		@context.show_page
	end

	def getch
	end

	def close
		@surface.write_to_png(@file + '.' + @ext) if(@ext == 'png')
		@surface.finish
	end
end

