# coding: utf-8

require 'gtk2'

#===============================================================================
#
#	Class: LegacyGraphics
#
class LegacyGraphics

	@@color = []
	pow = { '0' => 0, '1' => 65535 }; (0..7).each {|c|		# イロイッカイズツ
		grb = '%03b' % c; @@color << Gdk::Color.new(pow[grb[1]], pow[grb[0]], pow[grb[2]])
	}
	@@colormap = Gdk::Colormap.system
	@@color.each {|c| @@colormap.alloc_color(c, false, true) }

	def initialize(h_dmy, w_dmy, y_dmy, x_dmy, gy = 400, gx = 640, cy = 16, cx = 8, c = 0, out = {})
		@lx = 0
		@ly = 0

		@window = Gtk::Window.new
		@window.signal_connect('destroy') { Gtk.main_quit }
		@window.title = 'TrueLegacyGraphicsGtk2'
		@window.set_size_request(gx, gy)
		@window.set_app_paintable(true)
		@window.realize

		# http://ruby-gnome2.sourceforge.jp/hiki.cgi?Gdk%3A%3ADrawable
		@drawable = @window.window
		@gc = Gdk::GC.new(@drawable)

		@draw_procs = []
		fill(0, 0, gx - 1, gy - 1, c)
	end

	def pset(x, y, c)

		fill(x, y, x, y, c)
	end

	def locate(x, y)
		@tx = x * 8; @ty = y * 16
	end

	def print(mes)
		text(@tx, @ty, mes, 7)
	end

	def line(x1, y1, x2, y2, c, bf = '', w = 1, d = nil, a = nil)
		if(bf =~ /f/i)
			fill(x1, y1, x2, y2, c)
		elsif(bf =~ /b/i)
			# not implement yet
		else
			@lx = x2
			@ly = y2
			_line(x1, y1, x2, y2, c, w, d, a)
		end
	end

	def cline(x, y, c, w = 1, d = nil, a = nil)
			  x1 = @lx
			  y1 = @ly
		@lx = x2 = x
		@ly = y2 = y
		_line(x1, y1, x2, y2, c, w, d, a)
	end

	#---------------------------------------------------------------------------
	#
	#	_line
	#
	def _line(x1, y1, x2, y2, c, w, d, a)

		@draw_procs << Proc.new {
			@gc.set_foreground(@@color[c.class <= Numeric ? c : 7])
			@drawable.draw_line(@gc, x1, y1, x2, y2)
		}
		pset(x2, y2, c)
	end

	def circle(x, y, r, c)
		# not implement yet
	end

	def fill(x1, y1, x2, y2, c)

		@draw_procs << Proc.new {
			@gc.set_foreground(@@color[c.class <= Numeric ? c : 7])
			@drawable.draw_rectangle(@gc, true, x1, y1, x2 - x1 + 1, y2 - y1 + 1)
		}
	end

	def polygon(points, c, f = '')				# points = [[x1, y1], [x2, y2], [x3, y3]...]

		@draw_procs << Proc.new {
			@gc.set_foreground(@@color[c.class <= Numeric ? c : 7])
			@drawable.draw_polygon(@gc, f =~ /f/i, points)
		}
	end

	def put_image_png(x, y, file)

		@draw_procs << Proc.new {
			@drawable.draw_pixbuf(@gc, Gdk::Pixbuf.new(file), 0, 0, x, y, -1, -1, Gdk::RGB::DITHER_NONE, 0, 0)
		}
	end

	def text(x, y, s, c, ps = {})
#		{ 'font.family' => 'Helvetica', 'font.weight' => 400, 'font.size' => 10, 'layout.width' => 128, 'layout.alignment' => 'left', 'get_size' => true }

		# http://ruby-gnome2.sourceforge.jp/hiki.cgi?Pango%3A%3AFontDescription
		fontdesc = Pango::FontDescription.new
		fontdesc.family = ps[:font_family] || 'Helvetica'
		fontdesc.weight = ps[:font_weight] || Pango::WEIGHT_NORMAL # WEIGHT_NORMAL: 400, WEIGHT_BOLD: 700
		fontdesc.absolute_size = (ps[:font_size] || 10) * Pango::SCALE

		# http://ruby-gnome2.sourceforge.jp/hiki.cgi?Pango%3A%3ALayout
		layout = Pango::Layout.new(@window.pango_context)
		layout.font_description = fontdesc

		layout.width = (ps[:layout_width] || 128) * Pango::SCALE	# x ------- width(dot) ->|
		layout.alignment = Pango::Layout::ALIGN_LEFT				# |left....center...right|
		layout.alignment = Pango::Layout::ALIGN_CENTER	if(it = ps[:layout_alignment] and it =~ /center/i)
		layout.alignment = Pango::Layout::ALIGN_RIGHT	if(it = ps[:layout_alignment] and it =~ /right/i)
		layout.text = s

		ps[:get_size] and return(layout.pixel_size)
		@draw_procs << Proc.new {
			@gc.set_foreground(@@color[c.class <= Numeric ? c : 7])
			@drawable.draw_layout(@gc, x, y, layout)
		}
	end

	def rotate(r = nil, x = 0, y = 0)
		# not implement yet
		yield
	end

	def clear
	end

	def box(vchar_dmy, hchar_dmy)
		# not implement yet
	end

	def refresh
		@window.signal_connect('expose_event') {
			@draw_procs.each {|proc| proc.call }
		}
		@window.show_all
		Gtk.main
	end

	def getch
	end

	def close
	end
end

